"""
	Licensed Materials - Property of IBM Corp. 
IBM Rational Build Forge 
(c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.   

U.S. Government Users Restricted Rights - Use, duplication or disclosure restricted by 
GSA ADP Schedule Contract with IBM Corp. 

	
	File name: AdminHelper.py
	
	Contains utility methods for WebSphere Application Server
"""

from SystemUtils import *
from Logger import _Logger

import AdminControl
import AdminConfig
import AdminTask

############## CLASS :: _AdminHelper #####################
# This class provides helper methods for WAS			 #
# Administration										 #
#														#
##########################################################
class _AdminHelper:

	###########################################################
	## 
	###########################################################
	def __init__(self):
		self.LOGGER = _Logger("AdminHelper")
	#endDef
	
	############## buildScope ###############################
	#   Description:										#
	#	used to build scope path for getid calls		   #
	#													   #
	#   Inputs:											 #
	#   optDict = (first value return from getopt)		  #
	#########################################################
	def buildScope (self, optDict):
	
		# set the cellname
		cellname = AdminControl.getCell()
		scope = ''
		
		# get the variable map at the right level in  scope
		if (optDict['scope'] == 'cell'):
			scope = '/Cell:' + cellname + '/'
		
		elif (optDict['scope'] == 'cluster'):
			if (len(optDict['scopename']) < 1):
				print "You didn't declare clustername and yet the scope is cluster."
				sys.exit(1)
			#endIf
			scope = '/Cell:' + cellname + '/ServerCluster:' + optDict['scopename'] + '/'
		
		elif (optDict['scope'] == 'node'):
			if (len(optDict['scopename']) < 1):
				print "You didn't declare nodename and yet the scope is node."
				sys.exit(1)
			#endIf
			scope = '/Cell:' + cellname + '/Node:' + optDict['scopename'] + '/'
		
		elif (optDict['scope'] == 'server'):
			if (len(optDict['nodename']) < 1):
				print "You didn't declare nodename and yet the scope is servernode. A nodename must be declared to find the server."
				sys.exit(1)
			if (len(optDict['scopename']) < 1):
				print "You didn't declare servername and yet the scope is server."
				sys.exit(1)
				#endIf
			scope = '/Cell:' + cellname + '/Node:' + optDict['nodename'] + '/Server:' + optDict['scopename'] + '/'
		
		else:
			print "Scope: %s is invalid. Exiting!" % (optDict['scope'])
			sys.exit(1)
		#endIf
		return scope
	#endDef 

	############ isInScope(scope, oid) ########################
	# check if oid is in scope
	###############################################################
	def isInScope(self, scope, oid):
		if (scope == "node" and SystemUtils.regexp('servers', oid) == 1):
			return 0
		elif (scope == "cell" and SystemUtils.regexp('nodes', oid) == 1):
			return 0
		elif (scope == "cell" and SystemUtils.regexp('clusters', oid) == 1):
			return 0
		else:
			return 1
		#endif		
	#endDef

	"""
		Returns the lowest/deepest scope for oid.
	"""
	def getScope(self, oid):
		if (SystemUtils.regexp('servers', oid) == 1):
			return "server"
		elif (SystemUtils.regexp('nodes', oid) == 1):
			return "node"
		elif (SystemUtils.regexp('clusters', oid) == 1):
			return "cluster"
		else:
			return "cell"
		#endIf
	#endDef
	
	############ buildAttrs(params, paramList) ########################
	# get attributes for modify												  
	########################################################################
	def buildAttrs(self, params, paramList):
		attrs=[]
		for key in paramList:
			attrs.append([key, params[key]])
		#endFor	
		return attrs
	#endDef	

	########### syncNodes ################################
	# 	Description:									 #
	#		Used to Synchronize nodes					#
	#													 #
	#   Inputs:										  #
	#	nodes = (comma separated list of nodes)		  #
	#		ex: "server1,server2"						#	
	######################################################
	def syncNodes (self, nodes):
		for nodeName in nodes.split(","):
			print "checking for the existence of a NodeSync MBean on node "+nodeName
			try:
				nodeSync = AdminControl.completeObjectName("type=NodeSync,node="+nodeName+",*")
				if (len(nodeSync) == 0):
					print "Error -- NodeSync MBean not found for name "+nodeName
				#endIf 
				enabled = AdminControl.getAttribute(nodeSync, "serverStartupSyncEnabled")
				if (cmp(enabled, "false") == 0):
					print "Invoking synchronization for node " + nodeSync + \
						" because serverStartupSyncEnabled is set to false..."
					AdminControl.invoke(nodeSync, "sync")
					print "Done with synchronization."
				#endIf 
			except:
				print "--## Failed to get nodesync bean: skipping sync for this node. ##--"
				continue
			#endTry
		#endFor 
		AdminConfig.save()
	#endDef 

	########### syncCell #################################
	# 	Description:									 #
	#		Used to Syncronize nodes					 #
	######################################################
	def syncCell (self):
		nodes = []
		for node in self.listNodes():
			for server in AdminConfig.list('Server', node).split(newline):
				## server will be "" if there are no servers on this node
				if (len (server) > 0):
					type = AdminConfig.showAttribute(server, 'serverType')
					if (type != 'DEPLOYMENT_MANAGER' and type != 'WEB_SERVER'):
						nodeName = AdminConfig.showAttribute(node, "name")
						print "--## %s contains a non-deploymentManger and will be synced ##--" % (nodeName)
						nodes.append(nodeName)
						break
					#endIf
				#endIf
			#endFor
		#endFor
		if (len(nodes) > 0):
			self.syncNodes(','.join(nodes))
		else:
			print "No syncable nodes found: exiting!"
		#endIf
	#endDef
	
	def saveAndSyncCell(self):
		# save config
		print "Saving Config..."
		AdminConfig.save( )

		#Syncronize the Cell
		print "Synchronizing Nodes..."
		self.syncCell( )	
	#endDef
	
	############ listNodes() ##############################
	# This simply lists all of the nodes				  #
	#######################################################
	def listNodes(self):
		nodeList = AdminConfig.list("Node").split(newline)
		return nodeList
	#endDef
	
	############ listServers() ############################
	# This simply lists all of the servers				#
	#######################################################
	def listServers(self):
		serverList = AdminConfig.list("Server").split(newline)
		return serverList
	#endDef
	
	############ listServers() ############################
	# This simply lists all of the servers				#
	#######################################################
	def listServersOnNode(self, nodeName):
		cellName = AdminControl.getCell()
		nodeId = AdminConfig.getid("/Cell:" + cellName + "/Node:" + nodeName)
		serverList = AdminConfig.list("Server", nodeId).split(newline)
		return serverList
	#endDef
	
	########### checkServerStarted  ######################
	# 	Description:									 #
	#		Used to verify if a server is started		#
	#													 #
	#   Inputs:										  #
	#	nodeName = (name of node server is on)		   #
	#	serverName = (name of server to verify)		  #	
	#													 #
	#   Returns:										 #
	#	0 = not started								  #
	#	1 = started									  #
	######################################################
	def checkServerStarted (self, serverName, nodeName):

		print "checkServerStarted: nodeName=" + nodeName + " serverName=" + serverName + " ..."
		serverID = ""
		retries = -1
		while ((len(serverID) == 0)  and (retries < 3)):
			retries = retries+1
			print "checkServerStarted: server state unknown, retries="+str(retries)+" ..."
			SystemUtils.sleepDelay(1)
			try:
				_excp_ = 0
				serverID = AdminControl.completeObjectName("node="+nodeName+",name="+serverName+",type=Server,*")
				excp_ = 0 #reset (in case of nested exceptions)
			except:
				_type_, _value_, _tbck_ = sys.exc_info()
				_excp_ = 1
			#endTry
			temp = _excp_
			if (temp != 0):
				print "checkServerStarted AdminControl exception="+temp+" serverID="+serverID
			#endIf
			print "checkServerStarted: serverID="+serverID+"  retries="+str(retries)+" ..."
		#endWhile
		if (retries > 0):
			print "checkServerStarted: "+nodeName+" "+serverName+" wasn't started, status retries="+str(retries)
		#endIf
		if (serverID == ""):
			print "WAS: Node: "+nodeName+" Server: "+serverName+" IS NOT STARTED."
			return 0
		#endIf
		state = AdminControl.getAttribute(serverID, "state")
		state = state.upper()
		if (state == "STARTED"):
			print "checkServerStarted: "+nodeName+" "+serverName+" state="+state
			return 1
		else:
			print nodeName+" "+serverName+" INCORRECT state="+state
			return 0
		#endElse
		print "checkServerStarted: DONE."
	#endDef

	########### startServer ##############################
	# 	Description:									 #
	#		Used to start a server					   #
	#													 #
	#   Inputs:										  #
	#	nodeName = (name of node server is on)		   #
	#	serverName = (name of server to start)		   #	
	######################################################
	def startServer (self, serverName, nodeName):
		print "startServer: nodeName="+nodeName+" serverName="+serverName+" ..."
		started = ""
		try:
				_excp_ = 0
				if (self.checkServerStarted(serverName, nodeName) != 1):
					 started = AdminControl.startServer(serverName, nodeName, 100000)
				_excp_ = 0 #reset (in case of nested exceptions)
		except:
				_type_, _value_, _tbck_ = sys.exc_info()
				_excp_ = 1
		#endTry
		temp = _excp_
		print "startServer: errorcode="+str(temp)+" started="+started+" ..."
		retries = 0
		while ((temp > 0)  and (retries < 5)):
				retries = retries+1
				print "start failed exception="+temp+" for "+nodeName+" "+serverName+", retries="+str(retries)+" ..."
				try:
						_excp_ = 0
						started = AdminControl.startServer(serverName, nodeName, 30000)
						_excp_ = 0 #reset (in case of nested exceptions)
				except:
						_type_, _value_, _tbck_ = sys.exc_info()
						_excp_ = 1
				#endTry
				temp = _excp_
				print "temp="+temp+" started="+started+" ..."
		#endWhile
		if (self.checkServerStarted(serverName, nodeName) == 1):
				print "startServer: DONE."
	#endDef

	########### stopServer ###############################
	# 	Description:									 #
	#		Used to stop a server						#
	#													 #
	#   Inputs:										  #
	#	nodeName = (name of node server is on)		   #
	#	serverName = (name of server to start)		   #	
	######################################################
	def stopServer (self, serverName, nodeName):
		if (self.checkServerStarted(serverName, nodeName) == 1):
			print "stopServer: nodeName="+nodeName+" serverName="+serverName+" ..."
			AdminControl.stopServer(serverName, nodeName)
		else:
			print "stopServer: nodeName="+nodeName+" serverName="+serverName+" is already stopped..."
	#endDef
	
	########### startStopServersOnNode ###################
	# 	Description:									 #
	#		Used to start/stop servers on Node		   #
	#													 #
	#   Inputs:										  #
	#	action = [start, stop, ripplestart]			  #
	#	serverList = (name of servers to start/stop)	 #	
	######################################################
	def startStopServersOnNode(self, action, nodeName):
		serverList = self.listServersOnNode(nodeName)
		
		for serverName in serverList:
			if (action == "start"):
				self.startServer(serverName, nodeName)
			if ( action == "stop" ):
				self.stopServer(serverName, nodeName)
			if ( action == "ripplestart" ):
				self.stopServer(serverName, nodeName)
				self.startServer(serverName, nodeName)
			#endIf
		#endFor
	#endDef

	
#endClass

## define global singletons
AdminHelper = _AdminHelper()	

